/*
   hash.c -- handles:
     (non-Tcl) procedure lookups for msg/dcc/file commands
     (Tcl) binding internal procedures to msg/dcc/file commands

   dprintf'ized, 15nov95
*/

#if HAVE_CONFIG_H
#include <config.h>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#ifdef TCL
#include "../lush.h"
#endif
#include "eggdrop.h"
#include "cmdt.h"
#include "hash.h"
#include "proto.h"


extern struct dcc_t dcc[];
extern int dcc_total;
extern int serv;
extern char cx_file[];
extern int cx_line;
extern char SBUF[];

/* new hashing function */
void gotcmd(nick,from,msg)
char *nick,*from,*msg;
{
  char *code,hand[41],s[121],*total;
#ifndef TCL
  int atr,cnt,ans,ok,x,f,i;
#endif
  context;
  code=SBUF; total=&SBUF[512];
  sprintf(s,"%s!%s",nick,from); strcpy(total,msg); rmspace(msg);
  nsplit(code,msg); get_handle_by_host(hand,s); rmspace(msg);
#ifdef TCL
  if (check_tcl_msg(code,nick,from,hand,msg)) return;
  check_tcl_msgm(code,nick,from,hand,msg);
  log(LOG_MSGS,"[%s!%s] %s",nick,from,total);
#else
  cnt=0; ok=0; ans=(-1); f=0;
  atr=get_attr_handle(hand);
  i=0; while ((C_msg[i].access != -1) && (!f)) {
    if (strncasecmp(code,C_msg[i].name,strlen(code))==0) {
      /* found a match */
      x=C_msg[i].access; ok=1;
      if (x&NEED_PARTY) if (!(atr & USER_PARTY)) ok=0;
      if (x&NEED_XFER) if (!(atr & USER_XFER)) ok=0;
      if (x&NEED_OP) if (!(atr & USER_OP)) ok=0;
      if (x&NEED_MASTER) if (!(atr & USER_MASTER)) ok=0;
      if (atr & USER_MASTER) ok=1;
      if (x&NEED_OWNER) if (!(atr & USER_OWNER)) ok=0;
      if (ok) {
	ans=i; cnt++;
	x=strcasecmp(code,C_msg[i].name);
	if (x==0) {
	  /* perfect match: this is the one */
	  f=1; cnt=1;
	}
      }
    }
    i++;
  }
  if ((ans>=0) && (cnt==1)) {
    ok=(C_msg[ans].func)(hand,nick,from,msg);
    if (!ok) ans=(-1);
  }
  if (ans<0) log(LOG_MSGS,"[%s!%s] %s",nick,from,total);
#endif
}

/* for dcc commands -- hash the function */
int got_dcc_cmd(idx,msg)
int idx; char *msg;
{
  char *total,*code;
#ifndef TCL
  int i,f,x,ok,ans,cnt,atr;
#endif
  context;
  code=SBUF; total=&SBUF[512];
  strcpy(total,msg); rmspace(msg); nsplit(code,msg); rmspace(msg);
#ifdef TCL
  return check_tcl_dcc(code,idx,msg);
#else
  cnt=0; ok=0; ans=(-1); f=0;
  i=0; while ((C_dcc[i].access != -1) && (!f)) {
    if (strncasecmp(code,C_dcc[i].name,strlen(code))==0) {
      /* found a match */
      x=C_dcc[i].access; ok=1; atr=get_attr_handle(dcc[idx].nick);
      if (x&NEED_PARTY) if (!(atr & USER_PARTY)) ok=0;
      if (x&NEED_XFER) if (!(atr & USER_XFER)) ok=0;
      if (x&NEED_OP) if (!(atr & USER_OP)) ok=0;
      if (x&NEED_MASTER) if (!(atr & USER_MASTER)) ok=0;
      if (atr & USER_MASTER) ok=1;
      if (x&NEED_OWNER) if (!(atr & USER_OWNER)) ok=0;
      if (ok) {
	ans=i; cnt++;
	x=strcasecmp(code,C_dcc[i].name);
	if (x==0) {
	  /* perfect match: this is the one */
	  f=1; cnt=1;
	}
      }
    }
    i++;
  }
  if ((ans>=0) && (cnt==1)) {
    if (C_dcc[ans].func == CMD_LEAVE) return 1;
    (C_dcc[ans].func)(idx,msg);
    return 0;
  }
  if (ans<0) dprintf(idx,"What?  Try '.help'\n");
  if (cnt>1) dprintf(idx,"Ambigious command.\n");
  return 0;
#endif
}

/* hash function for file area commands */
int got_files_cmd(idx,msg)
int idx; char *msg;
{
  char *total,*code;
#ifndef TCL
  int i,f,x,ok,ans,cnt,atr;
#endif
  context;
  code=SBUF; total=&SBUF[512];
  if (msg[0]=='.') strcpy(msg,&msg[1]);
  strcpy(total,msg); rmspace(msg); nsplit(code,msg); rmspace(msg);
#ifdef TCL
  return check_tcl_fil(code,idx,msg);
#else
  ans=(-1); cnt=0; f=0;
  i=0; while ((C_file[i].access != -1) && (!f)) {
    if (strncasecmp(code,C_file[i].name,strlen(code))==0) {
      /* found a match */
      x=C_file[i].access; ok=1; atr=get_attr_handle(dcc[idx].nick);
      if (x&NEED_PARTY) if (!(atr & USER_PARTY)) ok=0;
      if (x&NEED_XFER) if (!(atr & USER_XFER)) ok=0;
      if (x&NEED_OP) if (!(atr & USER_OP)) ok=0;
      if (x&NEED_MASTER) if (!(atr & USER_JANITOR)) ok=0;
      if (atr & USER_MASTER) ok=1;
      if (x&NEED_OWNER) if (!(atr & USER_OWNER)) ok=0;
      if (ok) {
	ans=i; cnt++;
	x=strcasecmp(code,C_file[i].name);
	if (x==0) {
	  /* perfect match: this is the one */
	  f=1; cnt=1;
	}
      }	
    }
    i++;
  }
  if ((ans>=0) && (cnt==1)) {
    if (C_file[ans].func == CMD_LEAVE) return 1;
    (C_file[ans].func)(idx,msg);
    return 0;
  }
  if (ans<0) dprintf(idx,"What?  Try '.help'\n");
  if (cnt>1) dprintf(idx,"Ambigious command.\n");
  return 0;
#endif
}

/* hash function for tandem bot commands */
int got_tandem(idx,msg)
int idx; char *msg;
{
  char *total,*code; int i,f;
  context;
  code=SBUF; total=&SBUF[512];
  strcpy(total,msg); nsplit(code,msg);
  f=0; i=0; while ((C_tand[i].access != -1) && (!f)) {
    if (strcasecmp(code,C_tand[i].name)==0) {
      /* found a match */
      (C_tand[i].func)(idx,msg); f=1;
    }
    i++;
  }
/*  if (!f) log(LOG_MISC,"Unknown tandem cmd: [%s] %s",dcc[idx].nick,total); */
  return 0;
}


#ifdef TCL
extern Tcl_HashTable H_msg, H_dcc, H_fil;
extern int hashtot;

#include "tclegg.h"

/* TCL bots use the TCL hash table for dcc and msg commands, so gotta */
/* copy the default hash tables into the TCL one */
void init_builtins()
{
  int i,j,flags,new; Tcl_HashTable *ht=NULL; Tcl_HashEntry *he;
  tcl_cmd_t *tt; cmd_t *cc=NULL;
  for (j=0; j<3; j++) {
    switch(j) {
    case 0:
      ht=&H_msg; cc=C_msg; break;
    case 1:
      ht=&H_dcc; cc=C_dcc; break;
    case 2:
      ht=&H_fil; cc=C_file; break;
    }
    i=0; while (cc[i].access != (-1)) {
      flags=0;
      if (cc[i].access & NEED_PARTY) flags|=USER_PARTY;
      if (cc[i].access & NEED_XFER) flags|=USER_XFER;
      if (cc[i].access & NEED_OP) flags|=USER_OP;
      if (j==2) {
	if (cc[i].access & NEED_MASTER) flags|=USER_JANITOR;
      }
      else {
	if (cc[i].access & NEED_MASTER) flags|=USER_MASTER;
      }
      if (cc[i].access & NEED_OWNER) flags|=USER_OWNER;
      tt=(tcl_cmd_t *)tclcmd_alloc(strlen(cc[i].name)+2);
      tt->flags_needed=flags; tt->next=NULL;
      strcpy(tt->func_name,"*"); strcat(tt->func_name,cc[i].name);
      he=Tcl_CreateHashEntry(ht,cc[i].name,&new);
      if (!new) {
	/* append old entry */
	tcl_cmd_t *ttx=(tcl_cmd_t *)Tcl_GetHashValue(he);
	Tcl_DeleteHashEntry(he);
	tt->next=ttx;
      }
      Tcl_SetHashValue(he,tt);
      i++;
    }
  }
}

#endif
